@@ -3,13 +3,17 @@ import sys
 import numpy
 import time
 from argparse import ArgumentParser
+import pkg_resources
+from subprocess import check_call
 
 from pyshortcuts import make_shortcut, ico_ext
 
 from .site_config import icondir, home_dir, uname
+from .site_config import (extras_wxgraph, extras_qtgraph,
+                          extras_epics, extras_xrd, extras_doc)
 from .shell import shell
 from .xmlrpc_server import larch_server_cli
-from .version import __version__, __date__, make_banner
+from .version import __version__, __date__, make_banner, check_larchversion
 
 HAS_CONDA = os.path.exists(os.path.join(sys.prefix, 'conda-meta'))
 
@@ -26,6 +30,16 @@ call %~dp0%activate base
 
 """
 
+def install_extras(package_set):
+    all_packages = set([pkg.key for pkg in pkg_resources.working_set])
+    missing = package_set - all_packages
+    if missing:
+        command = [sys.executable, '-m', 'pip', 'install', *missing]
+        check_call(command)
+
+def update_larch():
+    check_call([sys.executable, '-m', 'pip', 'install', '--upgrade', 'xraylarch'])
+
 def use_mpl_wxagg():
     """import matplotlib, set backend to wxAgg"""
     if HAS_WXPYTHON:
@@ -118,7 +132,6 @@ def make_desktop_shortcuts():
     for app in APPS:
         app.create_shortcut()
 
-
 def make_cli(description='run larch program', filedesc='data file'):
     usage = "usage: %prog [options] file"
     parser = ArgumentParser(description=description)
@@ -130,28 +143,37 @@ def make_cli(description='run larch program', filedesc='data file'):
 def run_gse_mapviewer():
     """Mapviewer"""
     use_mpl_wxagg()
+    install_extras(extras_wxgraph)
+    install_extras(extras_epics)
+    install_extras(extras_xrd)
+    vinfo = check_larchversion()
     from larch.wxmap import MapViewer
     kwargs = make_cli(description="Larch's XRM Map Viewer and Analysis Program",
                       filedesc='XRM Map File (.h5)')
-    MapViewer(**kwargs).MainLoop()
+    MapViewer(version_info=vinfo, **kwargs).MainLoop()
 
 def run_gse_dtcorrect():
     """GSE DT Correct """
     use_mpl_wxagg()
+    install_extras(extras_wxgraph)
+    install_extras(extras_epics)
     from larch.wxmap import DTViewer
     DTViewer().MainLoop()
 
 def run_xas_viewer():
     """XAS Viewer """
     use_mpl_wxagg()
+    install_extras(extras_wxgraph)
+    vinfo = check_larchversion()
     from larch.wxxas import XASViewer
-    kwargs = make_cli(description="Larch's XAS Viewer and Analysis Program",
-                      filedesc='XAS data file or Project File (.dat, .prj)')
-    XASViewer(**kwargs).MainLoop()
+    kwargs = make_cli(description="Larch's XAS Viewer and Analysis Program")
+    XASViewer(version_info=vinfo, **kwargs).MainLoop()
 
 def run_xrfdisplay():
     """ XRF Display"""
     use_mpl_wxagg()
+    install_extras(extras_wxgraph)
+    install_extras(extras_epics)
     from larch.wxlib.xrfdisplay import XRFApp
     kwargs = make_cli(description="Larch's XRF Viewer and Analysis Program",
                     filedesc='MCA File (.mca)')
@@ -160,6 +182,8 @@ def run_xrfdisplay():
 def run_xrfdisplay_epics():
     """XRF Display for Epics Detectors"""
     use_mpl_wxagg()
+    install_extras(extras_wxgraph)
+    install_extras(extras_epics)
     from larch.epics import EpicsXRFApp
     EpicsXRFApp().MainLoop()
 
@@ -172,6 +196,9 @@ def run_xrd1d_viewer():
 def run_xrd2d_viewer():
     """XRD Display for 2D patternss"""
     use_mpl_wxagg()
+    install_extras(extras_wxgraph)
+    install_extras(extras_epics)
+    install_extras(extras_xrd)
     from larch.wxxrd import XRD2DViewer
     XRD2DViewer().MainLoop()
 
@@ -221,6 +248,9 @@ def run_larch():
     parser.add_argument("-m", "--makeicons", dest="makeicons", action="store_true",
                         default=False, help="create desktop icons")
 
+    parser.add_argument('-u', '--update', dest='update', action='store_true',
+                        default=False, help='update larch to the latest version')
+
     parser.add_argument("-r", "--remote", dest="server_mode", action="store_true",
                         default=False, help="run in remote server mode")
 
@@ -233,19 +263,31 @@ def run_larch():
     args = parser.parse_args()
     if args.version:
         print(make_banner())
+        vinfo = check_larchversion()
+        if vinfo.update_available:
+            print(vinfo.message)
         return
 
     with_wx = HAS_WXPYTHON and (not args.nowx)
 
     # create desktop icons
     if args.makeicons:
+        install_extras(extras_wxgraph)
         make_desktop_shortcuts()
         return
 
+    # run updates
+    if args.update:
+        update_larch()
+        return
+
     # run in server mode
     if args.server_mode:
         if with_wx:
             use_mpl_wxagg()
+        vinfo = check_larchversion()
+        if vinfo.update_available:
+            print(vinfo.message)
 
         from larch.xmlrpc_server import LarchServer
         server = LarchServer(host='localhost', port=int(args.port))
@@ -254,6 +296,9 @@ def run_larch():
     # run wx Larch GUI
     elif args.wxgui:
         use_mpl_wxagg()
+        install_extras(extras_wxgraph)
+        install_extras(extras_epics)
+        install_extras(extras_xrd)
         from larch.wxlib.larchframe import LarchApp
         LarchApp().MainLoop()
 
@@ -261,6 +306,13 @@ def run_larch():
     else:
         if with_wx:
             use_mpl_wxagg()
+            install_extras(extras_wxgraph)
+        install_extras(extras_epics)
+        install_extras(extras_xrd)
+        vinfo = check_larchversion()
+        if vinfo.update_available:
+            print(vinfo.message)
+
         cli = shell(quiet=args.quiet, with_wx=with_wx)
         # execute scripts listed on command-line
         if args.scripts is not None:
