@@ -37,15 +37,16 @@ if HAS_WXPYTHON:
     from . import larchfilling
     from . import readlinetextctrl
 
-    from wxutils import (set_sizer, pack, SetTip, Font, HLine, Check, MenuItem,
-                         Popup, is_wxPhoenix, RIGHT, LEFT, CEN ,
-                         LTEXT, FRAMESTYLE, hms, DateTimeCtrl, Button,
-                         ToggleButton, BitmapButton, Choice, YesNo, SimpleText,
-                         TextCtrl, LabeledTextCtrl, HyperText, EditableListBox,
-                         get_icon, GUIColors, OkCancel, FileOpen, FileSave,
-                         SelectWorkdir, fix_filename, SavedParameterDialog,
-                         FileCheckList, FileDropTarget, GridPanel, RowPanel,
-                         make_steps, set_float, FloatCtrl)
+    from wxutils import (set_sizer, pack, SetTip, Font, HLine, Check,
+                         MenuItem, Popup, RIGHT, LEFT, CEN , LTEXT,
+                         FRAMESTYLE, hms, DateTimeCtrl, Button,
+                         ToggleButton, BitmapButton, Choice, YesNo,
+                         SimpleText, TextCtrl, LabeledTextCtrl, HyperText,
+                         get_icon, GUIColors, OkCancel,
+                         FileOpen, FileSave, SelectWorkdir, fix_filename,
+                         SavedParameterDialog, FileCheckList,
+                         FileDropTarget, GridPanel, RowPanel, make_steps,
+                         set_float, FloatCtrl)
 
     from .floats import NumericCombo, FloatSpin, FloatSpinWithPin
 
@@ -56,8 +57,8 @@ if HAS_WXPYTHON:
     from .athena_importer import AthenaImporter
     from .specfile_importer import SpecfileImporter
     from .reportframe import ReportFrame
-    from .gui_utils import gcd, databrowser, fileprompt, wx_update
-
+    from .gui_utils import gcd, databrowser, fileprompt, wx_update, EditableListBox
+    from .larch_updater import LarchUpdaterDialog
     from .parameter import ParameterWidgets, ParameterPanel
     from .periodictable import PeriodicTablePanel
 
@@ -106,6 +107,8 @@ if HAS_WXPYTHON:
                                     plot_chie=xafsplots.plot_chie,
                                     plot_chik=xafsplots.plot_chik,
                                     plot_chir=xafsplots.plot_chir,
+                                    plot_chiq=xafsplots.plot_chiq,
+                                    plot_wavelet=xafsplots.plot_wavelet,
                                     plot_chifit=xafsplots.plot_chifit,
                                     plot_path_k=xafsplots.plot_path_k,
                                     plot_path_r=xafsplots.plot_path_r,
