@@ -21,7 +21,9 @@ from wxutils import (SimpleText, FloatCtrl, GUIColors, Button, Choice,
 import larch
 from larch import Group
 from larch.utils.strutils import fix_varname
+from larch.xafs.xafsutils import guess_energy_units
 from larch.io import look_for_nans, is_specfile, open_specfile
+from larch.utils.physical_constants import PLANCK_HC, DEG2RAD, PI
 
 CEN |=  wx.ALL
 FNB_STYLE = fnb.FNB_NO_X_BUTTON|fnb.FNB_SMART_TABS
@@ -34,7 +36,8 @@ ARR_OPS = ('+', '-', '*', '/')
 YERR_OPS = ('Constant', 'Sqrt(Y)', 'Array')
 CONV_OPS  = ('Lorenztian', 'Gaussian')
 
-DATATYPES = ('raw', 'xas')
+XDATATYPES = ('raw', 'xas')
+ENUNITS_TYPES = ('eV', 'keV', 'degrees', 'not energy')
 
 class AddColumnsFrame(wx.Frame):
     """Add Column Labels for a larch grouop"""
@@ -316,7 +319,8 @@ class EditColumnFrame(wx.Frame) :
 
 class SpecfileImporter(wx.Frame) :
     """Column Data File, select columns"""
-    def __init__(self, parent, filename=None, read_ok_cb=None):
+    def __init__(self, parent, filename=None, last_array_sel=None, _larch=None,
+                 read_ok_cb=None):
         if not is_specfile(filename):
             title = "Not a Specfile: %s" % filename
             message = "Error reading %s as a Specfile" % filename
@@ -328,7 +332,7 @@ class SpecfileImporter(wx.Frame) :
         path, fname = os.path.split(filename)
         self.filename = fname
         self.extra_sums = {}
-
+        self._larch = _larch
         self.specfile = open_specfile(filename)
         self.scans = []
         curscan = None
@@ -350,14 +354,18 @@ class SpecfileImporter(wx.Frame) :
 
         if self.workgroup.datatype is None:
             self.workgroup.datatype = 'raw'
-            if 'en' in self.curscan.axis.lower():
-                self.workgroup.datatype = 'xas'
+            for arrlab in arr_labels[:4]:
+                if 'ener' in arrlab.lower():
+                    self.workgroup.datatype = 'xas'
 
         self.read_ok_cb = read_ok_cb
-        self.array_sel = {'scan': '',
-                          'xpop': '',  'xarr': None,
-                          'ypop': '',  'yop': '/',
-                          'yarr1': None, 'yarr2': None}
+
+        self.array_sel = dict(xarr=self.curscan.axis.lower(), yarr1=None, yarr2=None, yop='/',
+                              ypop='', monod=3.1355316, en_units='eV',
+                              yerror=YERR_OPS[0], yerr_val=1, yerr_arr=None)
+
+        if last_array_sel is not None:
+            self.array_sel.update(last_array_sel)
 
         if self.array_sel['yarr2'] is None and 'i0' in arr_labels:
             self.array_sel['yarr2'] = 'i0'
@@ -368,12 +376,11 @@ class SpecfileImporter(wx.Frame) :
             elif 'i1' in arr_labels:
                 self.array_sel['yarr1'] = 'i1'
 
-        wx.Frame.__init__(self, None, -1,
-                          'Build Arrays for %s' % filename,
+        wx.Frame.__init__(self, None, -1, f'Build Arrays for {filename:s}',
                           style=FRAMESTYLE)
 
         self.SetMinSize((750, 550))
-        self.SetSize((800, 650))
+        self.SetSize((850, 650))
         self.colors = GUIColors()
 
         x0, y0 = parent.GetPosition()
@@ -421,93 +428,140 @@ class SpecfileImporter(wx.Frame) :
         yarr_labels = self.yarr_labels = arr_labels + ['1.0', '0.0', '']
         xarr_labels = self.xarr_labels = arr_labels + ['_index']
 
-        self.xarr   = Choice(panel, choices=xarr_labels, action=self.onUpdate, size=(150, -1))
+        self.xarr   = Choice(panel, choices=xarr_labels, action=self.onXSelect, size=(150, -1))
         self.yarr1  = Choice(panel, choices=yarr_labels, action=self.onUpdate, size=(150, -1))
         self.yarr2  = Choice(panel, choices=yarr_labels, action=self.onUpdate, size=(150, -1))
         self.yerr_arr = Choice(panel, choices=yarr_labels, action=self.onUpdate, size=(150, -1))
         self.yerr_arr.Disable()
 
-        self.xpop = Choice(panel, choices=XPRE_OPS, action=self.onUpdate, size=(120, -1))
-        self.ypop = Choice(panel, choices=YPRE_OPS, action=self.onUpdate, size=(120, -1))
-
-        self.datatype = Choice(panel, choices=DATATYPES, action=self.onUpdate, size=(120, -1))
+        self.datatype = Choice(panel, choices=XDATATYPES, action=self.onUpdate, size=(150, -1))
         self.datatype.SetStringSelection(self.workgroup.datatype)
 
+        self.en_units = Choice(panel, choices=ENUNITS_TYPES, action=self.onEnUnitsSelect,
+                               size=(150, -1))
+
+        self.ypop = Choice(panel, choices=YPRE_OPS, action=self.onUpdate, size=(150, -1))
         self.yop =  Choice(panel, choices=ARR_OPS, action=self.onUpdate, size=(50, -1))
+        self.yerr_op = Choice(panel, choices=YERR_OPS, action=self.onYerrChoice, size=(150, -1))
+        
+        self.yerr_val = FloatCtrl(panel, value=1, precision=4, size=(90, -1))
+        self.monod_val  = FloatCtrl(panel, value=3.1355316, precision=7, size=(90, -1))
+        
+        xlab = SimpleText(panel, ' X array: ')
+        ylab = SimpleText(panel, ' Y array: ')
+        units_lab = SimpleText(panel, '  Units:  ')
+        yerr_lab = SimpleText(panel, ' Yerror: ')
+        dtype_lab = SimpleText(panel, ' Data Type: ')
+        monod_lab = SimpleText(panel, ' Mono D spacing (Ang): ')
+        yerrval_lab = SimpleText(panel, ' Value:')
 
-        self.yerr_op = Choice(panel, choices=YERR_OPS, action=self.onYerrChoice, size=(120, -1))
-        self.yerr_op.SetSelection(0)
-
-        self.yerr_const = FloatCtrl(panel, value=1, precision=4, size=(90, -1))
-        ylab = SimpleText(panel, 'Y = ')
-        xlab = SimpleText(panel, 'X = ')
-        yerr_lab = SimpleText(panel, 'Yerror = ')
+        self.ysuf = SimpleText(panel, '')
         self.message = SimpleText(panel, '', font=Font(11),
                            colour=self.colors.title, style=LEFT)
 
-        self.xpop.SetStringSelection(self.array_sel['xpop'])
         self.ypop.SetStringSelection(self.array_sel['ypop'])
         self.yop.SetStringSelection(self.array_sel['yop'])
+        self.monod_val.SetValue(self.array_sel['monod'])
+        self.monod_val.SetAction(self.onUpdate)
+        
+        self.monod_val.Enable(self.array_sel['en_units'].startswith('deg'))
+        self.en_units.SetStringSelection(self.array_sel['en_units'])
+        self.yerr_op.SetStringSelection(self.array_sel['yerror'])
+        self.yerr_val.SetValue(self.array_sel['yerr_val'])
         if '(' in self.array_sel['ypop']:
             self.ysuf.SetLabel(')')
 
-        ixsel, iysel, iy2sel = 0, 1, len(yarr_labels)-1
+        ixsel, iysel, iy2sel, iyesel = 0, 1, len(yarr_labels)-1,  len(yarr_labels)-1
         if self.array_sel['xarr'] in xarr_labels:
             ixsel = xarr_labels.index(self.array_sel['xarr'])
         if self.array_sel['yarr1'] in arr_labels:
             iysel = arr_labels.index(self.array_sel['yarr1'])
         if self.array_sel['yarr2'] in yarr_labels:
             iy2sel = yarr_labels.index(self.array_sel['yarr2'])
+        if self.array_sel['yerr_arr'] in yarr_labels:
+            iyesel = yarr_labels.index(self.array_sel['yerr_arr'])
         self.xarr.SetSelection(ixsel)
         self.yarr1.SetSelection(iysel)
         self.yarr2.SetSelection(iy2sel)
+        self.yerr_arr.SetSelection(iyesel)
 
         sizer = wx.GridBagSizer(2, 2)
         ir = 0
         sizer.Add(self.title,     (ir, 0), (1, 7), LEFT, 5)
 
-        # sizer.Add(scanlabel,     (ir, 0), (1, 1), LEFT, 0)
-        # sizer.Add(self.wid_scan, (ir, 1), (1, 3), LEFT, 0)
-
         ir += 1
-        sizer.Add(self.wid_scantitle,  (ir, 0), (1, 2), LEFT, 0)
-        sizer.Add(self.wid_scantime,   (ir, 2), (1, 2), LEFT, 0)
+        sizer.Add(self.wid_scantitle,  (ir, 0), (1, 3), LEFT, 0)
+        sizer.Add(self.wid_scantime,   (ir, 3), (1, 2), LEFT, 0)
 
 
         ir += 1
         sizer.Add(xlab,      (ir, 0), (1, 1), LEFT, 0)
-        sizer.Add(self.xpop, (ir, 1), (1, 1), CEN, 0)
-        sizer.Add(self.xarr, (ir, 2), (1, 1), CEN, 0)
+        sizer.Add(self.xarr,  (ir, 1), (1, 1), LEFT, 0)
+        sizer.Add(units_lab,     (ir, 2), (1, 2), RIGHT, 0)
+        sizer.Add(self.en_units,  (ir, 4), (1, 2), LEFT, 0)
+
+        ir += 1
+        sizer.Add(dtype_lab,          (ir, 0), (1, 1), LEFT, 0)
+        sizer.Add(self.datatype,      (ir, 1), (1, 1), LEFT, 0)
+        sizer.Add(monod_lab,          (ir, 2), (1, 2), RIGHT, 0)
+        sizer.Add(self.monod_val,     (ir, 4), (1, 1), LEFT, 0)
 
         ir += 1
         sizer.Add(ylab,       (ir, 0), (1, 1), LEFT, 0)
-        sizer.Add(self.ypop,  (ir, 1), (1, 1), CEN, 0)
-        sizer.Add(self.yarr1, (ir, 2), (1, 1), CEN, 0)
-        sizer.Add(self.yop,   (ir, 3), (1, 1), CEN, 0)
-        sizer.Add(self.yarr2, (ir, 4), (1, 1), CEN, 0)
+        sizer.Add(self.ypop,  (ir, 1), (1, 1), LEFT, 0)
+        sizer.Add(self.yarr1, (ir, 2), (1, 1), LEFT, 0)
+        sizer.Add(self.yop,   (ir, 3), (1, 1), RIGHT, 0)
+        sizer.Add(self.yarr2, (ir, 4), (1, 1), LEFT, 0)
+        sizer.Add(self.ysuf,  (ir, 5), (1, 1), LEFT, 0)        
 
         ir += 1
         sizer.Add(yerr_lab,      (ir, 0), (1, 1), LEFT, 0)
-        sizer.Add(self.yerr_op,  (ir, 1), (1, 1), CEN, 0)
-        sizer.Add(self.yerr_arr, (ir, 2), (1, 1), CEN, 0)
-        sizer.Add(SimpleText(panel, 'Value:'), (ir, 3), (1, 1), CEN, 0)
-        sizer.Add(self.yerr_const, (ir, 4), (1, 2), CEN, 0)
-
-        ir += 1
-        sizer.Add(SimpleText(panel, 'Data Type:'),  (ir, 0), (1, 1), LEFT, 0)
-        sizer.Add(self.datatype,                    (ir, 1), (1, 2), LEFT, 0)
+        sizer.Add(self.yerr_op,  (ir, 1), (1, 1), LEFT, 0)
+        sizer.Add(self.yerr_arr, (ir, 2), (1, 1), LEFT, 0)
+        sizer.Add(yerrval_lab,   (ir, 3), (1, 1), RIGHT, 0)
+        sizer.Add(self.yerr_val, (ir, 4), (1, 2), LEFT, 0)
 
         ir += 1
         sizer.Add(self.message,                     (ir, 0), (1, 4), LEFT, 0)
-
         pack(panel, sizer)
 
+        self.nb = fnb.FlatNotebook(rightpanel, -1, agwStyle=FNB_STYLE)
+        self.nb.SetTabAreaColour(wx.Colour(248,248,240))
+        self.nb.SetActiveTabColour(wx.Colour(254,254,195))
+        self.nb.SetNonActiveTabTextColour(wx.Colour(40,40,180))
+        self.nb.SetActiveTabTextColour(wx.Colour(80,0,0))
+
         self.plotpanel = PlotPanel(rightpanel, messenger=self.plot_messages)
-        self.plotpanel.SetMinSize((200, 200))
+        self.plotpanel.SetMinSize((300, 250))
+
+        shead = wx.Panel(rightpanel)
+        self.scanheader = wx.TextCtrl(shead, style=wx.TE_MULTILINE|wx.TE_READONLY,
+                                      size=(400, 250))
+        self.scanheader.SetValue('\n'.join(self.curscan.scan_header))
+        self.scanheader.SetFont(Font(10))        
+        textsizer = wx.BoxSizer(wx.VERTICAL)
+        textsizer.Add(self.scanheader, 1, LEFT|wx.GROW, 1)
+        pack(shead, textsizer)
+
+        
+        fhead = wx.Panel(rightpanel)
+        self.fileheader = wx.TextCtrl(fhead, style=wx.TE_MULTILINE|wx.TE_READONLY,
+                                      size=(400, 250))
+        self.fileheader.SetValue('\n'.join(self.curscan.file_header))
+        self.fileheader.SetFont(Font(10))        
+        textsizer = wx.BoxSizer(wx.VERTICAL)
+        textsizer.Add(self.fileheader, 1, LEFT|wx.GROW, 1)
+        pack(fhead, textsizer)
+
+
+
+        self.nb.AddPage(fhead, ' File Header ', True)
+        self.nb.AddPage(shead, ' Scan Header ', True)
+        self.nb.AddPage(self.plotpanel, ' Plot of Selected Arrays ', True)
 
         sizer = wx.BoxSizer(wx.VERTICAL)
         sizer.Add(panel, 0, LEFT|wx.GROW, 1)
-        sizer.Add(self.plotpanel, 1, LEFT|wx.GROW|wx.ALL, 1)
+        sizer.Add(self.nb, 1, LEFT|wx.GROW|wx.ALL, 1)
         pack(rightpanel, sizer)
 
         splitter.SplitVertically(leftpanel, rightpanel, 1)
@@ -517,11 +571,36 @@ class SpecfileImporter(wx.Frame) :
         for i in range(len(statusbar_fields)):
             self.statusbar.SetStatusText(statusbar_fields[i], i)
 
-        self.SetSize(self.GetBestSize())
+        self.set_energy_units()
+        csize = self.GetSize()
+        bsize = self.GetBestSize()
+        if bsize[0] > csize[0]: csize[0] = bsize[0]
+        if bsize[1] > csize[1]: csize[1] = bsize[1]
+        self.SetSize(csize)
         self.Show()
         self.Raise()
         self.onUpdate(self)
 
+    def set_energy_units(self):
+        ix  = self.xarr.GetSelection()
+        xname = self.xarr.GetStringSelection()
+        rdata = self.curscan.data
+        try:
+            ncol, npts = rdata.shape
+        except:
+            self.statusbar.SetStatusText(f"Warning: Could not read data for scan '{self.curscan.title:s}'")
+            
+        workgroup = self.workgroup
+        if xname.startswith('_index') or ix >= ncol:
+            workgroup.xdat = 1.0*np.arange(npts)
+        else:
+            workgroup.xdat = 1.0*rdata[ix, :]
+        eguess =  guess_energy_units(workgroup.xdat)
+        if eguess.startswith('eV'):
+            self.en_units.SetStringSelection('eV')
+        elif eguess.startswith('keV'):
+            self.en_units.SetStringSelection('keV')
+
     def onScanSelect(self, event=None):
         try:
             scan_desc = event.GetString()
@@ -534,7 +613,6 @@ class SpecfileImporter(wx.Frame) :
             slist.append(scan_desc)
         self.scanlist.SetCheckedStrings(slist)
 
-
         self.wid_scantitle.SetLabel("  %s" % self.curscan.title)
         self.wid_scantime.SetLabel(self.curscan.timestring)
 
@@ -571,9 +649,10 @@ class SpecfileImporter(wx.Frame) :
         self.workgroup.datatype = 'xas' if 'en' in xsel else 'raw'
         self.datatype.SetStringSelection(self.workgroup.datatype)
 
+        self.scanheader.SetValue('\n'.join(self.curscan.scan_header))
+        self.set_energy_units()
         self.onUpdate()
 
-
     def show_subframe(self, name, frameclass, **opts):
         shown = False
         if name in self.subframes:
@@ -644,12 +723,22 @@ class SpecfileImporter(wx.Frame) :
             else:
                 return
 
-        yerr_op = self.yerr_op.GetStringSelection().lower()
+        en_units = self.en_units.GetStringSelection()
+        dspace   = float(self.monod_val.GetValue())
+        xarr     = self.xarr.GetStringSelection()
+        yarr1    = self.yarr1.GetStringSelection()
+        yarr2    = self.yarr2.GetStringSelection()
+        ypop     = self.ypop.GetStringSelection()
+        yop      = self.yop.GetStringSelection()
+        yerr_op = self.yerr_op.GetStringSelection()
+        yerr_arr = self.yerr_arr.GetStringSelection()        
+        yerr_idx = self.yerr_arr.GetSelection()        
+        yerr_val = self.yerr_val.GetValue()
         yerr_expr = '1'
         if yerr_op.startswith('const'):
-            yerr_expr = "%f" % self.yerr_const.GetValue()
-        elif yerr_op.startswith('array'):
-            yerr_expr = '%%s.data[%i, :]' % self.yerr_arr.GetSelection()
+            yerr_expr = "%f" % yerr_val
+        elif yerr_op.lower().startswith('array'):
+            yerr_expr = '%%s.data[%i, :]' % yerr_idx
         elif yerr_op.startswith('sqrt'):
             yerr_expr = 'sqrt(%s.ydat)'
         self.expressions['yerr'] = yerr_expr
@@ -672,7 +761,16 @@ class SpecfileImporter(wx.Frame) :
             val = getattr(self.workgroup, attr)
             buff.append("{group}.%s = '%s'" % (attr, val))
 
-        for aname in ('xdat', 'ydat', 'yerr'):
+        expr = self.expressions['xdat'].replace('%s', '{group:s}')
+        if en_units.startswith('deg'):
+            buff.append(f"mono_dspace = {dspace:.9f}")
+            buff.append(f"{{group}}.xdat = PLANCK_HC/(2*mono_dspace*sin(DEG2RAD*({expr:s})))")
+        elif en_units.startswith('keV'):
+            buff.append(f"{{group}}.xdat = 1000.0*{expr:s}")
+        else:
+            buff.append(f"{{group}}.xdat = {expr:s}")
+
+        for aname in ('ydat', 'yerr'):
             expr = self.expressions[aname].replace('%s', '{group:s}')
             buff.append("{group}.%s = %s" % (aname, expr))
 
@@ -684,8 +782,20 @@ class SpecfileImporter(wx.Frame) :
             buff.append("{group}.scale = 1./({group}.ydat.ptp()+1.e-16)")
         script = "\n".join(buff)
 
+        self.array_sel['xarr'] = xarr
+        self.array_sel['yarr1'] = yarr1
+        self.array_sel['yarr2'] = yarr2
+        self.array_sel['yop'] = yop
+        self.array_sel['ypop'] = ypop
+        self.array_sel['yerror'] = yerr_op
+        self.array_sel['yerr_val'] = yerr_val
+        self.array_sel['yerr_arr'] = yerr_arr
+        self.array_sel['monod'] = dspace
+        self.array_sel['en_units'] = en_units
+
         if self.read_ok_cb is not None:
-            self.read_ok_cb(script, self.path, scanlist)
+            self.read_ok_cb(script, self.path, scanlist,
+                            array_sel=self.array_sel)
 
         for f in self.subframes.values():
             try:
@@ -706,13 +816,44 @@ class SpecfileImporter(wx.Frame) :
     def onYerrChoice(self, evt=None):
         yerr_choice = evt.GetString()
         self.yerr_arr.Disable()
-        self.yerr_const.Disable()
+        self.yerr_val.Disable()
         if 'const' in yerr_choice.lower():
-            self.yerr_const.Enable()
+            self.yerr_val.Enable()
         elif 'array' in yerr_choice.lower():
             self.yerr_arr.Enable()
         self.onUpdate()
 
+    def onXSelect(self, evt=None):
+        ix  = self.xarr.GetSelection()
+        xname = self.xarr.GetStringSelection()
+
+        workgroup = self.workgroup
+        rdata = self.curscan.data
+        ncol, npts = rdata.shape
+        if xname.startswith('_index') or ix >= ncol:
+            workgroup.xdat = 1.0*np.arange(npts)
+        else:
+            workgroup.xdat = 1.0*rdata[ix, :]
+
+        self.monod_val.Disable()
+        if self.datatype.GetStringSelection().strip().lower() == 'raw':
+            self.en_units.SetSelection(4)
+        else:
+            eguess = guess_energy_units(workgroup.xdat)
+            if eguess.startswith('keV'):
+                self.en_units.SetSelection(1)
+            elif eguess.startswith('deg'):
+                self.en_units.SetSelection(2)
+                self.monod_val.Enable()
+            else:
+                self.en_units.SetSelection(0)
+                
+        self.onUpdate()
+
+    def onEnUnitsSelect(self, evt=None):
+        self.monod_val.Enable(self.en_units.GetStringSelection().startswith('deg'))
+        self.onUpdate()
+
 
     def onUpdate(self, value=None, evt=None):
         """column selections changed calc xdat and ydat"""
@@ -739,9 +880,19 @@ class SpecfileImporter(wx.Frame) :
             xname = '_index'
             exprs['xdat'] = 'arange(%i)' % npts
         else:
-            workgroup.xdat = rdata[ix, :]
+            workgroup.xdat = 1.0*rdata[ix, :]
             exprs['xdat'] = '%%s.data[%i, : ]' % ix
 
+        xlabel = xname
+        en_units = self.en_units.GetStringSelection()
+        if en_units.startswith('deg'):
+            dspace = float(self.monod_val.GetValue())
+            workgroup.xdat = PLANCK_HC/(2*dspace*np.sin(DEG2RAD*workgroup.xdat))
+            xlabel = xname + ' (eV)'
+        elif en_units.startswith('keV'):
+            workgroup.xdat *= 1000.0
+            xlabel = xname + ' (eV)'
+        
         workgroup.datatype = self.datatype.GetStringSelection().strip().lower()
 
         def pre_op(opwid, arr):
@@ -756,13 +907,7 @@ class SpecfileImporter(wx.Frame) :
                 arr[np.where(np.isnan(arr))] = 0
             return suf, opstr, arr
 
-        try:
-            xsuf, xpop, workgroup.xdat = pre_op(self.xpop, workgroup.xdat)
-            exprs['xdat'] = '%s%s%s' % (xpop, exprs['xdat'], xsuf)
-        except:
-            return
 
-        xlabel = xname
         ylabel = yname1
         if len(yname2) == 0:
             yname2 = '1.0'
@@ -809,7 +954,7 @@ class SpecfileImporter(wx.Frame) :
         yerr_op = self.yerr_op.GetStringSelection().lower()
         exprs['yerr'] = '1'
         if yerr_op.startswith('const'):
-            yerr = self.yerr_const.GetValue()
+            yerr = self.yerr_val.GetValue()
             exprs['yerr'] = '%f' % yerr
         elif yerr_op.startswith('array'):
             iyerr = self.yerr_arr.GetSelection()
@@ -821,7 +966,7 @@ class SpecfileImporter(wx.Frame) :
 
 
         self.expressions = exprs
-        self.array_sel = {'xpop': xpop, 'xarr': xname,
+        self.array_sel = {'xarr': xname,
                           'ypop': ypop, 'yop': yop,
                           'yarr1': yname1, 'yarr2': yname2}
         try:
@@ -836,9 +981,8 @@ class SpecfileImporter(wx.Frame) :
             ((len(en) > 1000 or any(np.diff(en) < 0) or
               ((max(en)-min(en)) > 350 and
                (np.diff(en[:100]).mean() < 1.0))))):
-            self.message.SetLabel("Warning: XAS data may need to be rebinned!")
-        else:
-            self.message.SetLabel("")
+            self.statusbar.SetStatusText("Warning: XAS data may need to be rebinned!")
+
 
         workgroup.filename    = self.curscan.filename
         workgroup.npts        = npts
@@ -863,4 +1007,4 @@ class SpecfileImporter(wx.Frame) :
 
 
     def plot_messages(self, msg, panel=1):
-        self.SetStatusText(msg, panel)
+        self.statusbar.SetStatusText(msg, panel)
