@@ -21,6 +21,7 @@ Plotting macros for XAFS data sets and fits
 from numpy import gradient, ndarray, diff, where, arange, argmin
 from larch import Group
 from larch.math import (index_of, index_nearest, interp)
+from larch.xafs import cauchy_wavelet
 
 try:
     import wx
@@ -31,7 +32,7 @@ except ImportError:
 if HAS_WXPYTHON:
     from .plotter import (_getDisplay, _plot, _oplot, _newplot, _fitplot,
                           _plot_text, _plot_marker, _plot_arrow,
-                          _plot_axvline, _plot_axhline)
+                          _plot_axvline, _plot_axhline, _imshow)
 else:
     def nullfunc(*args, **kws): pass
 
@@ -71,7 +72,9 @@ plotlabels = Group(k       = r'$k \rm\,(\AA^{-1})$',
                    flat    = r'flattened $\mu(E)$',
                    deconv  = r'deconvolved $\mu(E)$',
                    dmude   = r'$d\mu(E)/dE$',
-                   dnormde   = r'$d\mu_{\rm norm}(E)/dE$',
+                   dnormde = r'$d\mu_{\rm norm}(E)/dE$',
+                   d2mude  = r'$d^2\mu(E)/dE^2$',
+                   d2normde= r'$d^2\mu_{\rm norm}(E)/dE^2$',
                    chie    = r'$\chi(E)$',
                    chikw   = r'$k^{{{0:g}}}\chi(k) \rm\,(\AA^{{-{0:g}}})$',
                    chir    = r'$\chi(R) \rm\,(\AA^{{-{0:g}}})$',
@@ -411,10 +414,9 @@ def plot_chik(dgroup, kweight=None, kmax=None, show_window=True,
     redraw(win=win, xmax=kmax, _larch=_larch)
 #enddef
 
-
 def plot_chir(dgroup, show_mag=True, show_real=False, show_imag=False,
-              rmax=None, label=None, title=None, new=True, delay_draw=False,
-              offset=0, win=1, _larch=None):
+              show_window=False, rmax=None, label=None, title=None,
+              new=True, delay_draw=False, offset=0, win=1, _larch=None):
     """
     plot_chir(dgroup, show_mag=True, show_real=False, show_imag=False,
               rmax=None, label=None, new=True, win=1)
@@ -427,6 +429,7 @@ def plot_chir(dgroup, show_mag=True, show_real=False, show_imag=False,
      show_mag     bool whether to plot |chi(R)| [True]
      show_real    bool whether to plot Re[chi(R)] [False]
      show_imag    bool whether to plot Im[chi(R)] [False]
+     show_window  bool whether to R-windw for back FT (will be scaled) [False]
      label        string for label [``None`` to use 'chir']
      title        string for plot titlel [None, may use filename if available]
      rmax         max R to show [None, end of data]
@@ -466,11 +469,113 @@ def plot_chir(dgroup, show_mag=True, show_real=False, show_imag=False,
     if show_imag:
         _plot(dgroup.r, dgroup.chir_im+offset, label='%s (imag)' % label, **opts)
     #endif
-    if show_mag or show_real or show_imag:
+    if show_window and hasattr(dgroup, 'rwin'):
+        rwin = dgroup.rwin * max(dgroup.chir_mag)
+        opts['zorder'] = 15
+        _plot(dgroup.r, rwin+offset, label='window',  **opts)
+    #endif
+
+    if show_mag or show_real or show_imag or show_window:
         redraw(win=win, xmax=rmax, _larch=_larch)
     #endif
 #enddef
 
+def plot_chiq(dgroup, kweight=None, kmax=None, show_chik=False, label=None,
+              title=None, new=True, delay_draw=False, offset=0, win=1,
+              _larch=None):
+    """
+    plot_chiq(dgroup, kweight=None, kmax=None, show_chik=False, label=None,
+              new=True, win=1)
+
+    Plot Fourier filtered chi(k), optionally with k-weighted chi(k) for XAFS data group
+
+    Arguments
+    ----------
+     dgroup       group of XAFS data after autobk() results (see Note 1)
+     kweight      k-weighting for plot [read from last xftf(), or 0]
+     kmax         max k to show [None, end of data]
+     show_chik    bool whether to also plot k-weighted chi(k) [False]
+     label        string for label [``None`` to use 'chi']
+     title        string for plot titlel [None, may use filename if available]
+     new          bool whether to start a new plot [True]
+     delay_draw   bool whether to delay draw until more traces are added [False]
+     offset       vertical offset to use for y-array [0]
+     win          integer plot window to use [1]
+
+    Notes
+    -----
+     1. The input data group must have the following attributes:
+         k, chi, kwin, filename
+    """
+    if kweight is None:
+        kweight = 0
+        xft = getattr(dgroup, 'xftf_details', None)
+        if xft is not None:
+            kweight = xft.call_args.get('kweight', 0)
+        #endif
+    #endif
+    nk = len(dgroup.k)
+    chiq = dgroup.chiq_re[:nk]
+    opts = dict(win=win, show_legend=True, delay_draw=True, linewidth=3, _larch=_larch)
+    if label is None:
+        label = 'chi(q) (filtered)'
+    #endif
+    title = _get_title(dgroup, title=title)
+    _plot(dgroup.k, chiq+offset, xlabel=plotlabels.k,
+         ylabel=plotlabels.chikw.format(kweight), title=title,
+         label=label, zorder=20, new=new, xmax=kmax, **opts)
+
+    if show_chik:
+        chik = dgroup.chi * dgroup.k ** kweight
+        _plot(dgroup.k, chik+offset, zorder=16, label='chi(k)',  **opts)
+    #endif
+    redraw(win=win, xmax=kmax, _larch=_larch)
+#enddef
+
+
+def plot_wavelet(dgroup, show_mag=True, show_real=False, show_imag=False,
+                 rmax=None, kmax=None, kweight=None, title=None, win=1, _larch=None):
+    """
+    plot_wavelet(dgroup, show_mag=True, show_real=False, show_imag=False,
+              rmax=None, kmax=None, kweight=None, title=None, win=1)
+
+    Plot wavelet for XAFS data group
+
+    Arguments
+    ----------
+     dgroup       group of XAFS data after xftf() results (see Note 1)
+     show_mag     bool whether to plot wavelet magnitude [True]
+     show_real    bool whether to plot real part of wavelet [False]
+     show_imag    bool whether to plot imaginary part of wavelet [False]
+     title        string for plot titlel [None, may use filename if available]
+     rmax         max R to show [None, end of data]
+     kmax         max k to show [None, end of data]
+     kweight      k-weight to use to construct wavelet [None, take from group]
+     win          integer image window to use [1]
+
+    Notes
+    -----
+     The wavelet will be performed 
+    """
+    if kweight is None:
+        kweight = dgroup.xftf_details.call_args['kweight']
+        
+    title = _get_title(dgroup, title=title)
+
+    opts = dict(win=win, title=title, x=dgroup.k, y=dgroup.r, xmax=kmax,
+                ymax=rmax, xlabel=plotlabels.k, ylabel=plotlabels.r,
+                show_axis=True, _larch=_larch)
+
+    cauchy_wavelet(dgroup, kweight=kweight)
+    if show_mag:
+        _imshow(dgroup.wcauchy_mag, **opts)
+    elif show_real:
+        _imshow(dgroup.wcauchy_real, **opts)        
+    elif show_imag:
+        _imshow(dgroup.wcauchy_imag, **opts)
+    #endif
+#enddef
+
 def plot_chifit(dataset, kmin=0, kmax=None, kweight=None, rmax=None,
                 show_mag=True, show_real=False, show_imag=False,
                 title=None, new=True, delay_draw=False, offset=0, win=1,
