@@ -4,7 +4,6 @@ import numpy as np
 np.seterr(all='ignore')
 
 from functools import partial
-from collections import OrderedDict
 import json
 
 import wx
@@ -608,7 +607,7 @@ class PrePeakPanel(TaskPanel):
                            title='Pre-edge Peak Analysis',
                            config=defaults, **kws)
 
-        self.fit_components = OrderedDict()
+        self.fit_components = {}
         self.user_added_params = None
 
         self.pick2_timer = wx.Timer(self)
@@ -627,6 +626,8 @@ class PrePeakPanel(TaskPanel):
             fname = self.controller.filelist.GetStringSelection()
             gname = self.controller.file_groups[fname]
             dgroup = self.controller.get_group(gname)
+            if not hasattr(dgroup, 'norm'):
+                self.xasmain.process_normalization(dgroup)
             self.fill_form(dgroup)
         except:
             pass # print(" Cannot Fill prepeak panel from group ")
@@ -768,6 +769,8 @@ class PrePeakPanel(TaskPanel):
 
     def fill_form(self, dat):
         if isinstance(dat, Group):
+            if not hasattr(dat, 'norm'):
+                self.xasmain.process_normalization(dat)
             # self.wids['ppeak_e0'].SetValue(dat.e0)
             if hasattr(dat, 'prepeaks'):
                 self.wids['ppeak_emin'].SetValue(dat.prepeaks.emin)
@@ -1014,7 +1017,7 @@ write_ascii('{savefile:s}', {gname:s}.energy, {gname:s}.norm, {gname:s}.prepeaks
                        SLabel("  Min", size=(60, -1)),
                        SLabel("  Max", size=(60, -1)),  SLabel(" Expression")))
 
-        parwids = OrderedDict()
+        parwids = {}
         parnames = sorted(minst.param_names)
 
         for a in minst._func_allargs:
@@ -1323,7 +1326,7 @@ write_ascii('{savefile:s}', {gname:s}.energy, {gname:s}.norm, {gname:s}.prepeaks
         for igroup, gname in enumerate(groups):
             dgroup = self.controller.get_group(gname)
             if not hasattr(dgroup, 'norm'):
-                self.xasmain.get_nbpage('xasnorm').process(dgroup)
+                self.xasmain.process_normalization(dgroup)
             self.build_fitmodel(gname)
             opts['group'] = opts['gname']
             self.larch_eval(COMMANDS['prepeaks_setup'].format(**opts))
@@ -1432,7 +1435,7 @@ write_ascii('{savefile:s}', {gname:s}.energy, {gname:s}.norm, {gname:s}.prepeaks
 
     def autosave_modelresult(self, result, fname=None):
         """autosave model result to user larch folder"""
-        confdir = os.path.join(site_config.usr_larchdir, 'xas_viewer')
+        confdir = os.path.join(site_config.user_larchdir, 'xas_viewer')
         if not os.path.exists(confdir):
             try:
                 os.makedirs(confdir)
