@@ -1,5 +1,5 @@
 import os
-from collections import namedtuple, OrderedDict
+from collections import namedtuple
 from functools import partial
 import numpy as np
 from lmfit import Parameters, minimize
@@ -16,15 +16,9 @@ from larch.wxlib import (GridPanel, BitmapButton, FloatCtrl, FloatSpin,
                          plotlabels)
 
 from larch.xafs.xafsutils  import etok, ktoe
+from larch.utils.physical_constants import PI, DEG2RAD, PLANCK_HC
 
-PI = np.pi
-DEG2RAD  = PI/180.0
-
-# Planck constant over 2 pi times c: 197.3269718 (0.0000044) MeV fm
-PLANCK_HC = 1973.269718 * 2 * PI # hc in eV * Ang = 12398.4193
-
-Plot_Choices = OrderedDict((('Normalized', 'norm'),
-                            ('Derivative', 'dmude')))
+Plot_Choices = {'Normalized': 'norm', 'Derivative': 'dmude'}
 
 ELEM_LIST = ('H', 'He', 'Li', 'Be', 'B', 'C', 'N', 'O', 'F', 'Ne', 'Na',
              'Mg', 'Al', 'Si', 'P', 'S', 'Cl', 'Ar', 'K', 'Ca', 'Sc', 'Ti',
@@ -166,7 +160,7 @@ class OverAbsorptionDialog(wx.Dialog):
         xdat, ydat = self.data
         dgroup = self.dgroup
         dgroup.xdat = dgroup.energy = xdat
-        self.parent.nb.pagelist[0].process(dgroup)
+        self.parent.process_normalization(dgroup)
         self.plot_results()
 
     def plot_results(self, event=None):
@@ -175,11 +169,14 @@ class OverAbsorptionDialog(wx.Dialog):
         dgroup = self.dgroup
         path, fname = os.path.split(dgroup.filename)
 
-
         opts = dict(linewidth=3, ylabel=plotlabels.norm,
                     xlabel=plotlabels.energy, delay_draw=True,
                     show_legend=True)
 
+        if self.controller.plot_erange is not None:
+            opts['xmin'] = dgroup.e0 + self.controller.plot_erange[0]
+            opts['xmax'] = dgroup.e0 + self.controller.plot_erange[1]
+
         ppanel.plot(dgroup.energy, dgroup.norm_corr, zorder=10, marker=None,
                     title='Over-absorption Correction:\n %s' % fname,
                     label='corrected', **opts)
@@ -362,7 +359,7 @@ overwriting current arrays''')
         xdat, ydat = self.data
         dgroup = self.dgroup
         dgroup.xdat = dgroup.energy = xdat
-        self.parent.nb.pagelist[0].process(dgroup)
+        self.parent.process_normalization(dgroup)
         self.plot_results()
 
     def on_apply_sel(self, event=None):
@@ -371,7 +368,7 @@ overwriting current arrays''')
             fname  = self.controller.file_groups[str(checked)]
             dgroup = self.controller.get_group(fname)
             dgroup.xdat = dgroup.energy = eshift + dgroup.energy[:]
-            self.parent.nb.pagelist[0].process(dgroup)
+            self.parent.process_normalization(dgroup)
 
     def on_saveas(self, event=None):
         wids = self.wids
@@ -404,14 +401,16 @@ overwriting current arrays''')
             ynew = np.gradient(ynew)/np.gradient(xnew)
             ylabel = plotlabels.dmude
 
-        opts = dict(xmin=xmin, xmax=xmax, linewidth=3,
-                    ylabel=ylabel, xlabel=plotlabels.energy,
-                    delay_draw=True, show_legend=True)
+        opts = dict(xmin=xmin, xmax=xmax, linewidth=3, ylabel=ylabel,
+                    xlabel=plotlabels.energy, show_legend=True)
+
+        if self.controller.plot_erange is not None:
+            opts['xmin'] = dgroup.e0 + self.controller.plot_erange[0]
+            opts['xmax'] = dgroup.e0 + self.controller.plot_erange[1]
 
         ppanel.plot(xnew, ynew, zorder=20, marker=None,
                     title='Energy Calibration:\n %s' % fname,
-                    label='shifted', **opts)
-
+                    label='shifted', delay_draw=True, **opts)
 
         xold, yold = self.dgroup.energy, self.dgroup.norm
         if use_deriv:
@@ -429,14 +428,12 @@ overwriting current arrays''')
                          marker=None, label=refgroup.filename, **opts)
 
         axv_opts = dict(ymin=0.05, ymax=0.95, linewidth=2.0, alpha=0.5,
-                        zorder=1, label='_nolegend_')
-
+                         zorder=1, label='_nolegend_')
         color1 = ppanel.conf.traces[0].color
         color2 = ppanel.conf.traces[1].color
         ppanel.axes.axvline(e0_new, color=color1, **axv_opts)
         ppanel.axes.axvline(e0_old, color=color2, **axv_opts)
         ppanel.canvas.draw()
-        ppanel.conf.draw_legend(show=True)
 
     def GetResponse(self):
         raise AttributError("use as non-modal dialog!")
@@ -475,11 +472,9 @@ class RebinDataDialog(wx.Dialog):
                              **opts)
         pre1 = 10.0*(1+int((xmin-e0val)/10.0))
         wids['pre1'] = FloatCtrl(panel, value=pre1,  **opts)
-        wids['pre2'] = FloatCtrl(panel, value=-20, **opts)
-
+        wids['pre2'] = FloatCtrl(panel, value=-15, **opts)
         wids['xanes1'] = FloatCtrl(panel, value=-15,  **opts)
         wids['xanes2'] = FloatCtrl(panel, value=15, **opts)
-
         wids['exafs1'] = FloatCtrl(panel, value=etok(15),  **opts)
         wids['exafs2'] = FloatCtrl(panel, value=etok(xmax-e0val), **opts)
 
@@ -558,7 +553,7 @@ class RebinDataDialog(wx.Dialog):
         ngroup.mu     = ngroup.ydat = ydat
 
         ngroup.delta_mu = getattr(ngroup, 'yerr', 1.0)
-        self.parent.nb.pagelist[0].process(ngroup)
+        self.parent.process_normalization(dgroup)
         self.parent.onNewGroup(ngroup)
 
     def on_groupchoice(self, event=None):
@@ -607,7 +602,7 @@ class RebinDataDialog(wx.Dialog):
         dgroup = self.dgroup
         dgroup.energy = dgroup.xdat = xdat
         dgroup.mu     = dgroup.ydat = ydat
-        self.parent.nb.pagelist[0].process(dgroup)
+        self.parent.process_normalization(dgroup)
         self.plot_results()
 
     def on_done(self, event=None):
@@ -615,20 +610,24 @@ class RebinDataDialog(wx.Dialog):
 
     def plot_results(self):
         ppanel = self.controller.get_display(stacked=False).panel
-        ppanel.oplot
         xnew, ynew, yerr, e0 = self.data
         dgroup = self.dgroup
         path, fname = os.path.split(dgroup.filename)
 
+        opts = {}
+        if self.controller.plot_erange is not None:
+            opts['xmin'] = dgroup.e0 + self.controller.plot_erange[0]
+            opts['xmax'] = dgroup.e0 + self.controller.plot_erange[1]
+
         ppanel.plot(xnew, ynew, zorder=20, delay_draw=True, marker='square',
                     linewidth=3, title='Enegy rebinning:\n %s' % fname,
                     label='rebinned', xlabel=plotlabels.energy,
-                    ylabel=plotlabels.mu)
+                    ylabel=plotlabels.mu, **opts)
 
         xold, yold = self.dgroup.energy, self.dgroup.mu
         ppanel.oplot(xold, yold, zorder=10, delay_draw=False,
                      marker='o', markersize=4, linewidth=2.0,
-                     label='original', show_legend=True)
+                     label='original', show_legend=True, **opts)
         ppanel.canvas.draw()
 
     def GetResponse(self):
@@ -733,7 +732,7 @@ class SmoothDataDialog(wx.Dialog):
         xdat, ydat = self.data
         ngroup.energy = ngroup.xdat = xdat
         ngroup.mu     = ngroup.ydat = ydat
-        self.parent.nb.pagelist[0].process(ngroup)
+        self.parent.process_normalization(ngroup)
         self.parent.onNewGroup(ngroup)
 
     def on_groupchoice(self, event=None):
@@ -785,7 +784,7 @@ class SmoothDataDialog(wx.Dialog):
         dgroup = self.dgroup
         dgroup.energy = xdat
         dgroup.mu     = ydat
-        self.parent.nb.pagelist[0].process(dgroup)
+        self.parent.process_normalization(dgroup)
         self.plot_results()
 
     def on_done(self, event=None):
@@ -793,20 +792,24 @@ class SmoothDataDialog(wx.Dialog):
 
     def plot_results(self):
         ppanel = self.controller.get_display(stacked=False).panel
-        ppanel.oplot
         xnew, ynew = self.data
         dgroup = self.dgroup
         path, fname = os.path.split(dgroup.filename)
 
+        opts = {}
+        if self.controller.plot_erange is not None:
+            opts['xmin'] = dgroup.e0 + self.controller.plot_erange[0]
+            opts['xmax'] = dgroup.e0 + self.controller.plot_erange[1]
+
         ppanel.plot(xnew, ynew, zorder=20, delay_draw=True, marker=None,
                     linewidth=3, title='Smoothing:\n %s' % fname,
                     label='smoothed', xlabel=plotlabels.energy,
-                    ylabel=plotlabels.mu)
+                    ylabel=plotlabels.mu, **opts)
 
         xold, yold = self.dgroup.energy, self.dgroup.mu
         ppanel.oplot(xold, yold, zorder=10, delay_draw=False,
                      marker='o', markersize=4, linewidth=2.0,
-                     label='original', show_legend=True)
+                     label='original', show_legend=True, **opts)
         ppanel.canvas.draw()
 
     def GetResponse(self):
@@ -887,7 +890,7 @@ class DeconvolutionDialog(wx.Dialog):
         xdat, ydat = self.data
         ngroup.energy = ngroup.xdat = xdat
         ngroup.mu     = ngroup.ydat = ydat
-        self.parent.nb.pagelist[0].process(ngroup)
+        self.parent.process_normalization(ngroup)
         self.parent.onNewGroup(ngroup)
 
     def on_groupchoice(self, event=None):
@@ -913,25 +916,29 @@ class DeconvolutionDialog(wx.Dialog):
         dgroup = self.dgroup
         dgroup.energy = xdat
         dgroup.mu     = ydat
-        self.parent.nb.pagelist[0].process(dgroup)
+        self.parent.process_normalization(dgroup)
         self.plot_results()
 
     def plot_results(self):
         ppanel = self.controller.get_display(stacked=False).panel
-        ppanel.oplot
         xnew, ynew = self.data
         dgroup = self.dgroup
         path, fname = os.path.split(dgroup.filename)
 
+        opts = {}
+        if self.controller.plot_erange is not None:
+            opts['xmin'] = dgroup.e0 + self.controller.plot_erange[0]
+            opts['xmax'] = dgroup.e0 + self.controller.plot_erange[1]
+
         ppanel.plot(xnew, ynew, zorder=20, delay_draw=True, marker=None,
                     linewidth=3, title='Deconvolving:\n %s' % fname,
                     label='deconvolved', xlabel=plotlabels.energy,
-                    ylabel=plotlabels.mu)
+                    ylabel=plotlabels.mu, **opts)
 
         xold, yold = self.dgroup.energy, self.dgroup.norm
         ppanel.oplot(xold, yold, zorder=10, delay_draw=False,
                      marker='o', markersize=4, linewidth=2.0,
-                     label='original', show_legend=True)
+                     label='original', show_legend=True, **opts)
         ppanel.canvas.draw()
 
     def GetResponse(self):
@@ -1053,7 +1060,7 @@ clear undo history''')
         xdat, ydat = self.data[-1]
         ngroup.energy = ngroup.xdat = xdat
         ngroup.mu     = ngroup.ydat = ydat
-        self.parent.nb.pagelist[0].process(ngroup)
+        self.parent.process_normalization(ngroup)
         self.parent.onNewGroup(ngroup)
 
     def reset_data_history(self):
@@ -1115,26 +1122,29 @@ clear undo history''')
         dgroup.energy = dgroup.xdat = xdat
         dgroup.mu     = dgroup.ydat = ydat
         self.reset_data_history()
-        self.parent.nb.pagelist[0].process(dgroup)
+        self.parent.process_normalization(dgroup)
         self.plot_results()
 
     def plot_results(self):
         ppanel = self.controller.get_display(stacked=False).panel
-        ppanel.oplot
         xnew, ynew = self.data[-1]
         dgroup = self.dgroup
         path, fname = os.path.split(dgroup.filename)
 
+        opts = {}
+        if self.controller.plot_erange is not None:
+            opts['xmin'] = dgroup.e0 + self.controller.plot_erange[0]
+            opts['xmax'] = dgroup.e0 + self.controller.plot_erange[1]
         ppanel.plot(xnew, ynew, zorder=20, delay_draw=True, marker=None,
                     linewidth=3, title='De-glitching:\n %s' % fname,
                     label='current', xlabel=plotlabels.energy,
-                    ylabel=plotlabels.mu)
+                    ylabel=plotlabels.mu, **opts)
 
         if len(self.data) > 1:
             xold, yold = self.data[0]
             ppanel.oplot(xold, yold, zorder=10, delay_draw=False,
                          marker='o', markersize=4, linewidth=2.0,
-                         label='original', show_legend=True)
+                         label='original', show_legend=True, **opts)
         ppanel.canvas.draw()
         self.history_message.SetLabel('%i items in history' % (len(self.data)-1))
 
@@ -1178,7 +1188,7 @@ class SpectraCalcDialog(wx.Dialog):
         xmax = max(self.dgroup.energy)
         e0val = getattr(self.dgroup, 'e0', xmin)
 
-        wx.Dialog.__init__(self, parent, wx.ID_ANY, size=(550, 400),
+        wx.Dialog.__init__(self, parent, wx.ID_ANY, size=(475, 525),
                            title="Spectra Calculations: Add, Subtract Spectra")
 
         panel = GridPanel(self, ncols=3, nrows=4, pad=4, itemstyle=LEFT)
@@ -1398,10 +1408,10 @@ class ExportCSVDialog(wx.Dialog):
         title = "Export Selected Groups"
         wx.Dialog.__init__(self, parent, wx.ID_ANY, title=title)
 
-        self.ychoices = OrderedDict((('normalized mu(E)', 'norm'),
-                                     ('raw mu(E)', 'mu'),
-                                     ('flattened mu(E)', 'flat'),
-                                     ('d mu(E) / dE', 'dmude')))
+        self.ychoices = {'normalized mu(E)': 'norm',
+                         'raw mu(E)': 'mu',
+                         'flattened mu(E)': 'flat',
+                         'd mu(E) / dE': 'dmude'}
 
         default_fname = 'Data.csv'
         if len(groupnames) > 0:
