@@ -11,17 +11,16 @@ import numpy as np
 np.seterr(all='ignore')
 
 from functools import partial
-from collections import OrderedDict
+
 import wx
 import wx.lib.scrolledpanel as scrolled
 
 import wx.lib.mixins.inspection
+from wx.adv import AboutBox, AboutDialogInfo
 
 from wx.richtext import RichTextCtrl
 
-is_wxPhoenix = 'phoenix' in wx.PlatformInfo
-is_windows = platform.system().startswith('Windows')
-WX_DEBUG = True
+WX_DEBUG = False
 
 import larch
 from larch import Group
@@ -36,7 +35,7 @@ from larch.wxlib import (LarchFrame, ColumnDataFileFrame, AthenaImporter,
                          SetTip, get_icon, SimpleText, pack, Button, Popup,
                          HLine, FileSave, Choice, Check, MenuItem,
                          GUIColors, CEN, LEFT, FRAMESTYLE, Font, FONTSIZE,
-                         flatnotebook)
+                         flatnotebook, LarchUpdaterDialog)
 
 from larch.wxlib.plotter import _newplot, _plot, last_cursor_pos
 
@@ -50,6 +49,7 @@ from .lincombo_panel import LinearComboPanel
 from .pca_panel import PCAPanel
 from .exafs_panel import EXAFSPanel
 from .regress_panel import RegressionPanel
+from .xas_controller import XASController
 
 from .xas_dialogs import (MergeDialog, RenameDialog, RemoveDialog,
                           DeglitchDialog, ExportCSVDialog, RebinDataDialog,
@@ -65,18 +65,18 @@ from larch.xafs import pre_edge, pre_edge_baseline
 
 LEFT = wx.ALIGN_LEFT
 CEN |=  wx.ALL
-FILE_WILDCARDS = "Data Files(*.0*,*.dat,*.xdi,*.prj,*.spc)|*.0*;*.dat;*.DAT;*.xdi;*.prj;*.spc|All files (*.*)|*.*"
+FILE_WILDCARDS = "Data Files(*.0*,*.dat,*.xdi,*.prj,*.spc,*.hdf5)|*.0*;*.dat;*.DAT;*.xdi;*.prj;*.sp*c;*.h*5|All files (*.*)|*.*"
 
 ICON_FILE = 'onecone.ico'
 XASVIEW_SIZE = (950, 750)
 PLOTWIN_SIZE = (550, 550)
 
-NB_PANELS = OrderedDict((('Normalization', XASNormPanel),
-                         ('Pre-edge Peak', PrePeakPanel),
-                         ('PCA',  PCAPanel),
-                         ('Linear Combo', LinearComboPanel),
-                         ('Regression', RegressionPanel),
-                         ('EXAFS', EXAFSPanel)))
+NB_PANELS = {'Normalization': XASNormPanel,
+             'Pre-edge Peak': PrePeakPanel,
+             'PCA':  PCAPanel,
+             'Linear Combo': LinearComboPanel,
+             'Regression': RegressionPanel,
+             'EXAFS':  EXAFSPanel}
 
 QUIT_MESSAGE = '''Really Quit? You may want to save your project before quitting.
  This is not done automatically!'''
@@ -102,277 +102,18 @@ def assign_gsescan_groups(group):
 
     group.array_labels = labels
 
-class XASController():
-    """
-    class holding the Larch session and doing the processing work for XAS GUI
-    """
-    config_file = 'xas_viewer.conf'
-    def __init__(self, wxparent=None, _larch=None):
-        self.wxparent = wxparent
-        self.larch = _larch
-        if self.larch is None:
-            self.larch = larch.Interpreter()
-        self.filelist = None
-        self.file_groups = OrderedDict()
-        self.fit_opts = {}
-        self.group = None
-
-        self.groupname = None
-        self.report_frame = None
-        self.symtable = self.larch.symtable
-
-    def init_larch(self):
-        _larch = self.larch
-        old_config = read_config(self.config_file)
-
-        config = self.make_default_config()
-        for sname in config:
-            if old_config is not None and sname in old_config:
-                val = old_config[sname]
-                if isinstance(val, dict):
-                    config[sname].update(val)
-                else:
-                    config[sname] = val
-
-        for key, value in config.items():
-            setattr(_larch.symtable._sys.xas_viewer, key, value)
-        try:
-            os.chdir(config['workdir'])
-        except:
-            pass
-        self.wxparent.SetIcon(wx.Icon(self.get_iconfile(), wx.BITMAP_TYPE_ICO))
-
-    def make_default_config(self):
-        """ default config, probably called on first run of program"""
-        config = {'chdir_on_fileopen': True,
-                  'workdir': os.getcwd()}
-        return config
-
-    def get_config(self, key, default=None):
-        "get configuration setting"
-        confgroup = self.larch.symtable._sys.xas_viewer
-        return getattr(confgroup, key, default)
-
-    def save_config(self):
-        """save configuration"""
-        conf = group2dict(self.larch.symtable._sys.xas_viewer)
-        conf.pop('__name__')
-        save_config(self.config_file, conf)
-
-    def set_workdir(self):
-        self.larch.symtable._sys.xas_viewer.workdir = os.getcwd()
-
-    def get_iconfile(self):
-        return os.path.join(icondir, ICON_FILE)
-
-    def write_message(self, msg, panel=0):
-        """write a message to the Status Bar"""
-        self.wxparent.statusbar.SetStatusText(msg, panel)
-
-    def close_all_displays(self):
-        "close all displays, as at exit"
-        self.symtable._plotter.close_all_displays()
-
-    def get_display(self, win=1, stacked=False):
-        wintitle='Larch XAS Plot Window %i' % win
-        # if stacked:
-        #     win = 2
-        #    wintitle='Larch XAS Plot Window'
-        opts = dict(wintitle=wintitle, stacked=stacked, win=win,
-                    size=PLOTWIN_SIZE)
-        out = self.symtable._plotter.get_display(**opts)
-        if win > 1:
-            p1 = getattr(self.symtable._plotter, 'plot1', None)
-            if p1 is not None:
-                try:
-                    siz = p1.GetSize()
-                    pos = p1.GetPosition()
-                    pos[0] += int(siz[0]/4)
-                    pos[1] += int(siz[1]/4)
-                    out.SetSize(pos)
-                    if not stacked:
-                        out.SetSize(siz)
-                except Exception:
-                    pass
-        return out
-
-    def get_group(self, groupname=None):
-        if groupname is None:
-            groupname = self.groupname
-            if groupname is None:
-                return None
-        dgroup = getattr(self.symtable, groupname, None)
-        if dgroup is None and groupname in self.file_groups:
-            groupname = self.file_groups[groupname]
-            dgroup = getattr(self.symtable, groupname, None)
-        return dgroup
-
-    def filename2group(self, filename):
-        "convert filename (as displayed) to larch group"
-        return self.get_group(self.file_groups[str(filename)])
-
-    def merge_groups(self, grouplist, master=None, yarray='mu', outgroup=None):
-        """merge groups"""
-        cmd = """%s = merge_groups(%s, master=%s,
-        xarray='energy', yarray='%s', kind='cubic', trim=True)
-        """
-        glist = "[%s]" % (', '.join(grouplist))
-        outgroup = fix_varname(outgroup.lower())
-        if outgroup is None:
-            outgroup = 'merged'
-
-        outgroup = unique_name(outgroup, self.file_groups, max=1000)
-
-        cmd = cmd % (outgroup, glist, master, yarray)
-        self.larch.eval(cmd)
-
-        if master is None:
-            master = grouplist[0]
-        this = self.get_group(outgroup)
-        master = self.get_group(master)
-        if not hasattr(this, 'xasnorm_config'):
-            this.xasnorm_config = {}
-        this.xasnorm_config.update(master.xasnorm_config)
-        this.datatype = master.datatype
-        this.xdat = 1.0*this.energy
-        this.ydat = 1.0*getattr(this, yarray)
-        this.yerr =  getattr(this, 'd' + yarray, 1.0)
-        if yarray != 'mu':
-            this.mu = this.ydat
-        this.plot_xlabel = 'energy'
-        this.plot_ylabel = yarray
-        return outgroup
-
-    def copy_group(self, filename, new_filename=None):
-        """copy XAS group (by filename) to new group"""
-        groupname = self.file_groups[filename]
-        if not hasattr(self.larch.symtable, groupname):
-            return
-
-        ogroup = self.get_group(groupname)
-
-        ngroup = Group(datatype=ogroup.datatype,
-                       copied_from=groupname)
-
-        for attr in dir(ogroup):
-            do_copy = True
-            if attr in ('xdat', 'ydat', 'i0', 'data' 'yerr',
-                        'energy', 'mu'):
-                val = getattr(ogroup, attr)*1.0
-            elif attr in ('norm', 'flat', 'deriv', 'deconv',
-                          'post_edge', 'pre_edge', 'norm_mback',
-                          'norm_vict', 'norm_poly'):
-                do_copy = False
-            else:
-                try:
-                    val = copy.deepcopy(getattr(ogroup, attr))
-                except ValueError:
-                    do_copy = False
-            if do_copy:
-                setattr(ngroup, attr, val)
-
-        if new_filename is None:
-            new_filename = filename + '_1'
-        ngroup.filename = unique_name(new_filename, self.file_groups.keys())
-        ngroup.groupname = unique_name(groupname, self.file_groups.values())
-        setattr(self.larch.symtable, ngroup.groupname, ngroup)
-        return ngroup
-
-    def get_cursor(self, win=None):
-        """get last cursor from selected window"""
-        return last_cursor_pos(win=win, _larch=self.larch)
-
-    def plot_group(self, groupname=None, title=None, plot_yarrays=None,
-                   new=True, zoom_out=True, **kws):
-        ppanel = self.get_display(stacked=False).panel
-        newplot = ppanel.plot
-        oplot   = ppanel.oplot
-        plotcmd = oplot
-        viewlims = ppanel.get_viewlimits()
-        if new:
-            plotcmd = newplot
-
-        dgroup = self.get_group(groupname)
-        if not hasattr(dgroup, 'xdat'):
-            print("Cannot plot group ", groupname)
-
-        if ((getattr(dgroup, 'plot_yarrays', None) is None or
-             getattr(dgroup, 'energy', None) is None or
-             getattr(dgroup, 'mu', None) is None)):
-            self.process(dgroup)
-
-        if plot_yarrays is None and hasattr(dgroup, 'plot_yarrays'):
-            plot_yarrays = dgroup.plot_yarrays
-
-        popts = kws
-        path, fname = os.path.split(dgroup.filename)
-        if not 'label' in popts:
-            popts['label'] = dgroup.plot_ylabel
-        zoom_out = (zoom_out or
-                  min(dgroup.xdat) >= viewlims[1] or
-                  max(dgroup.xdat) <= viewlims[0] or
-                  min(dgroup.ydat) >= viewlims[3] or
-                  max(dgroup.ydat) <= viewlims[2])
-
-        if not zoom_out:
-            popts['xmin'] = viewlims[0]
-            popts['xmax'] = viewlims[1]
-            popts['ymin'] = viewlims[2]
-            popts['ymax'] = viewlims[3]
-
-        popts['xlabel'] = dgroup.plot_xlabel
-        popts['ylabel'] = dgroup.plot_ylabel
-        if getattr(dgroup, 'plot_y2label', None) is not None:
-            popts['y2label'] = dgroup.plot_y2label
-
-        plot_extras = None
-        if new:
-            if title is None:
-                title = fname
-            plot_extras = getattr(dgroup, 'plot_extras', None)
-
-        popts['title'] = title
-        if hasattr(dgroup, 'custom_plotopts'):
-            popts.update(dgroup.custom_plotopts)
-
-        narr = len(plot_yarrays) - 1
-        for i, pydat in enumerate(plot_yarrays):
-            yaname, yopts, yalabel = pydat
-            popts.update(yopts)
-            if yalabel is not None:
-                popts['label'] = yalabel
-            popts['delay_draw'] = (i != narr)
-
-            plotcmd(dgroup.xdat, getattr(dgroup, yaname), **popts)
-            plotcmd = oplot
-
-        if plot_extras is not None:
-            axes = ppanel.axes
-            for etype, x, y, opts in plot_extras:
-                if etype == 'marker':
-                    popts = {'marker': 'o', 'markersize': 4,
-                             'label': '_nolegend_',
-                             'markerfacecolor': 'red',
-                             'markeredgecolor': '#884444'}
-                    popts.update(opts)
-                    axes.plot([x], [y], **popts)
-                elif etype == 'vline':
-                    popts = {'ymin': 0, 'ymax': 1.0,
-                             'color': '#888888'}
-                    popts.update(opts)
-                    axes.axvline(x, **popts)
-        ppanel.canvas.draw()
-
 
 class XASFrame(wx.Frame):
     _about = """Larch XAS GUI: XAS Visualization and Analysis
 
     Matt Newville <newville @ cars.uchicago.edu>
     """
-    def __init__(self, parent=None, _larch=None, filename=None, **kws):
+    def __init__(self, parent=None, _larch=None, filename=None,
+                 version_info=None, **kws):
         wx.Frame.__init__(self, parent, -1, size=XASVIEW_SIZE, style=FRAMESTYLE)
 
-        self.last_array_sel = {}
+        self.last_array_sel_col = {}
+        self.last_array_sel_spec = {}
         self.paths2read = []
 
         title = "Larch XAS GUI: XAS Visualization and Analysis"
@@ -387,6 +128,9 @@ class XASFrame(wx.Frame):
         self.larch.symtable._sys.xas_viewer = Group()
 
         self.controller = XASController(wxparent=self, _larch=self.larch)
+        iconfile = os.path.join(icondir, ICON_FILE)
+        self.SetIcon(wx.Icon(iconfile, wx.BITMAP_TYPE_ICO))
+
         self.subframes = {}
         self.plotframe = None
         self.SetTitle(title)
@@ -403,6 +147,9 @@ class XASFrame(wx.Frame):
             self.statusbar.SetStatusText(statusbar_fields[i], i)
             self.current_filename = filename
         self.Show()
+        if version_info is not None:
+            if version_info.update_available:
+                self.onCheckforUpdates()
         if filename is not None:
             wx.CallAfter(self.onRead, filename)
 
@@ -461,6 +208,9 @@ class XASFrame(wx.Frame):
         splitter.SplitVertically(leftpanel, panel, 1)
         wx.CallAfter(self.init_larch)
 
+    def process_normalization(self, dgroup):
+        self.get_nbpage('xasnorm').process(dgroup, noskip=True)
+
     def get_nbpage(self, name):
         "get nb page by name"
         name = name.lower()
@@ -636,6 +386,13 @@ class XASFrame(wx.Frame):
         self.menubar.Append(data_menu, "Data")
         self.menubar.Append(ppeak_menu, "Pre-edge Peaks")
 
+        hmenu = wx.Menu()
+        MenuItem(self, hmenu, 'About XAS Viewer', 'About XAS Viewer',
+                 self.onAbout)
+        MenuItem(self, hmenu, 'Check for Updates', 'Check for Updates',
+                 self.onCheckforUpdates)
+
+        self.menubar.Append(hmenu, '&Help')
         # self.menubar.Append(ppeak_menu, "PreEdgePeaks")
         self.SetMenuBar(self.menubar)
         self.Bind(wx.EVT_CLOSE,  self.onClose)
@@ -687,7 +444,7 @@ class XASFrame(wx.Frame):
         self.save_athena_project(groups[0], groups, prompt=True)
 
     def onSaveProject(self, evt=None):
-        groups = [gname for gname in self.controller.file_groups]
+        groups = self.controller.filelist.GetItems()
         if len(groups) < 1:
             Popup(self, "No files selected to export to Athena",
                   "No files selected")
@@ -806,7 +563,7 @@ class XASFrame(wx.Frame):
             self.ShowFile(groupname=reproc_group, process=True)
 
     def onMergeData(self, event=None):
-        groups = OrderedDict()
+        groups = {}
         for checked in self.controller.filelist.GetCheckedStrings():
             cname = str(checked)
             groups[cname] = self.controller.file_groups[cname]
@@ -864,36 +621,60 @@ class XASFrame(wx.Frame):
         app = wx.GetApp()
         app.ShowInspectionTool()
 
-    def onAbout(self,evt):
-        dlg = wx.MessageDialog(self, self._about,
-                               "About Larch XAS GUI",
-                               wx.OK | wx.ICON_INFORMATION)
-        dlg.ShowModal()
-        dlg.Destroy()
 
-    def onClose(self, event):
-        dlg = QuitDialog(self)
+    def onAbout(self, event=None):
+        info = AboutDialogInfo()
+        info.SetName('XAS Viewer')
+        info.SetDescription('X-ray Absorption Visualization and Analysis')
+        info.SetVersion('Larch %s ' % larch.version.__version__)
+        info.AddDeveloper('Matthew Newville: newville at cars.uchicago.edu')
+        dlg = AboutBox(info)
+
+    def onCheckforUpdates(self, event=None):
+        dlg = LarchUpdaterDialog(self, caller='XAS Viewer')
         dlg.Raise()
         dlg.SetWindowStyle(wx.STAY_ON_TOP)
         res = dlg.GetResponse()
         dlg.Destroy()
-        if not res.ok:
-            return
+        if res.ok and res.run_updates:
+            from larch.apps import update_larch
+            update_larch()
+            self.onClose(event=event, prompt=False)
 
-        if res.save:
-            groups = [gname for gname in self.controller.file_groups]
-            if len(groups) > 0:
-                self.save_athena_project(groups[0], groups, prompt=True)
+    def onClose(self, event=None, prompt=True):
+        if prompt:
+            dlg = QuitDialog(self)
+            dlg.Raise()
+            dlg.SetWindowStyle(wx.STAY_ON_TOP)
+            res = dlg.GetResponse()
+            dlg.Destroy()
+            if not res.ok:
+                return
+
+            if res.save:
+                groups = [gname for gname in self.controller.file_groups]
+                if len(groups) > 0:
+                    self.save_athena_project(groups[0], groups, prompt=True)
 
         self.controller.save_config()
-        wx.CallAfter(self.controller.close_all_displays)
+        try:
+            self.controller.close_all_displays()
+        except Exception:
+            pass
 
         if self.larch_buffer is not None:
-            wx.CallAfter(self.larch_buffer.Destroy)
+            try:
+                self.larch_buffer.Destroy()
+            except Exception:
+                pass
 
         for name, wid in self.subframes.items():
             if hasattr(wid, 'Destroy'):
-                wx.CallAfter(wid.Destroy)
+                try:
+                    wid.Destroy()
+                except Exception:
+                    pass
+        time.sleep(0.05)
 
         self.Destroy()
 
@@ -912,7 +693,7 @@ class XASFrame(wx.Frame):
         dgroup = self.controller.get_group()
         self.show_subframe('readfile', ColumnDataFileFrame,
                            group=dgroup.raw,
-                           last_array_sel=self.last_array_sel,
+                           last_array_sel=self.last_array_sel_col,
                            _larch=self.larch,
                            read_ok_cb=partial(self.onRead_OK,
                                               overwrite=True))
@@ -962,16 +743,18 @@ class XASFrame(wx.Frame):
         elif is_specfile(path):
             self.show_subframe('spec_import', SpecfileImporter,
                                filename=path,
+                               _larch=self.larch_buffer.larchshell,
+                               last_array_sel=self.last_array_sel_spec,
                                read_ok_cb=self.onReadSpecfile_OK)
         # default to Column File
         else:
             self.show_subframe('readfile', ColumnDataFileFrame,
                                filename=path,
                                _larch=self.larch_buffer.larchshell,
-                               last_array_sel = self.last_array_sel,
+                               last_array_sel = self.last_array_sel_col,
                                read_ok_cb=self.onRead_OK)
 
-    def onReadSpecfile_OK(self, script, path, scanlist):
+    def onReadSpecfile_OK(self, script, path, scanlist, array_sel=None):
         """read groups from a list of scans from a specfile"""
         self.larch.eval("_specfile = specfile('{path:s}')".format(path=path))
         dgroup = None
@@ -980,14 +763,18 @@ class XASFrame(wx.Frame):
         cur_panel = self.nb.GetCurrentPage()
         cur_panel.skip_plotting = True
         symtable = self.larch.symtable
+        if array_sel is not None:
+            self.last_array_sel_spec = array_sel
+
         for scan in scanlist:
             gname = fix_varname("{:s}{:s}".format(fname[:6], scan))
             if hasattr(symtable, gname):
-                n = 0
-                while n < 99000:
-                    n += 1
-                    gname = fix_varname("{:s}_{%d}".format(gname, n))
-            
+                count, tname = 0, gname
+                while count < 1e7 and self.larch.symtable.has_group(tname):
+                    tname = gname + make_hashkey(length=7)
+                    count += 1
+                gname = tname
+
             cur_panel.skip_plotting = (scan == scanlist[-1])
             displayname = "%s_scan%s" % (fname, scan)
             if first_group is None:
@@ -1011,6 +798,7 @@ class XASFrame(wx.Frame):
 
         cur_panel = self.nb.GetCurrentPage()
         cur_panel.skip_plotting = True
+        labels = []
         for gname in namelist:
             cur_panel.skip_plotting = (gname == namelist[-1])
             this = getattr(self.larch.symtable._prj, gname)
@@ -1020,14 +808,15 @@ class XASFrame(wx.Frame):
                 while count < 1e7 and self.larch.symtable.has_group(gid):
                     gid = prefix + make_hashkey(length=7)
                     count += 1
-
+            label = getattr(this, 'label', gname)
+            labels.append(label)
             self.larch.eval(script.format(group=gid, prjgroup=gname))
-            dgroup = self.install_group(gid, gname, process=True, plot=False)
+            dgroup = self.install_group(gid, label, process=True, plot=False)
         self.larch.eval("del _prj")
         cur_panel.skip_plotting = False
 
-        if len(namelist) > 0:
-            gname = self.controller.file_groups[namelist[0]]
+        if len(labels) > 0:
+            gname = self.controller.file_groups[labels[0]]
             self.ShowFile(groupname=gname, process=True, plot=True)
         self.write_message("read %d datasets from %s" % (len(namelist), path))
 
@@ -1052,7 +841,7 @@ class XASFrame(wx.Frame):
 
         self.larch.eval(script.format(group=groupname, path=path))
         if array_sel is not None:
-            self.last_array_sel = array_sel
+            self.last_array_sel_col = array_sel
         self.install_group(groupname, filename, overwrite=overwrite)
 
         # check if rebin is needed
@@ -1124,15 +913,19 @@ class XASFrame(wx.Frame):
 
 
 class XASViewer(wx.App, wx.lib.mixins.inspection.InspectionMixin):
-    def __init__(self, filename=None, **kws):
+    def __init__(self, filename=None, description='Larch XAS Viewer',
+                 version_info=None,  **kws):
         self.filename = filename
+        self.description = description
+        self.version_info = version_info
         wx.App.__init__(self, **kws)
 
     def run(self):
         self.MainLoop()
 
     def createApp(self):
-        frame = XASFrame(filename=self.filename)
+        frame = XASFrame(filename=self.filename,
+                         version_info=self.version_info)
         self.SetTopWindow(frame)
 
     def OnInit(self):
