@@ -39,13 +39,24 @@ PlotOne_Choices = {'Raw \u03BC(E)': 'mu',
                    'MBACK + Poly Normalized': 'mback_poly',
                    'd\u03BC(E)/dE': 'dmude',
                    'Raw \u03BC(E) + d\u03BC(E)/dE': 'mu+dmude',
-                   'Normalized \u03BC(E) + d\u03BC(E)/dE': 'norm+dnormde'}
+                   'Normalized \u03BC(E) + d\u03BC(E)/dE': 'norm+dnormde',
+                   'd^2\u03BC(E)/dE^2': 'd2mude',
+                   'Normalized \u03BC(E) + d^2\u03BC(E)/dE^2': 'norm+d2normde',
+                   }
 
 PlotSel_Choices = {'Raw \u03BC(E)': 'mu',
                    'Normalized \u03BC(E)': 'norm',
                    'Flattened \u03BC(E)': 'flat',
                    'd\u03BC(E)/dE (raw)': 'dmude',
-                   'd\u03BC(E)/dE (normalized)': 'dnormde'}
+                   'd\u03BC(E)/dE (normalized)': 'dnormde',
+                   'd^2\u03BC(E)/dE^2': 'd2normde'}
+
+Plot_EnergyRanges = {'full E range': None,
+                     'E0 -20:+80eV':  (-20, 80),
+                     'E0 -30:+120eV': (-30, 120),
+                     'E0 -50:+250eV': (-50, 250),
+                     'E0 -100:+500eV': (-100, 500)}
+
 
 PlotOne_Choices_nonxas = {'Raw Data': 'mu',
                           'Scaled Data': 'norm',
@@ -80,12 +91,16 @@ class XASNormPanel(TaskPanel):
     def build_display(self):
         panel = self.panel
         self.wids = {}
-
+        self.last_plot_type = 'one'
         self.plotone_op = Choice(panel, choices=list(PlotOne_Choices.keys()),
                                  action=self.onPlotOne, size=(200, -1))
         self.plotsel_op = Choice(panel, choices=list(PlotSel_Choices.keys()),
                                  action=self.onPlotSel, size=(200, -1))
 
+        self.plot_erange = Choice(panel, choices=list(Plot_EnergyRanges.keys()),
+                                 action=self.onPlotEither, size=(120, -1))
+
+        self.plot_erange.SetSelection(0)
         self.plotone_op.SetSelection(1)
         self.plotsel_op.SetSelection(1)
 
@@ -144,8 +159,8 @@ class XASNormPanel(TaskPanel):
         atsyms = ['?'] + self.larch.symtable._xray._xraydb.atomic_symbols
         edges = ('K', 'L3', 'L2', 'L1', 'M5')
 
-        self.wids['atsym'] = Choice(panel, choices=atsyms, size=(75, -1))
-        self.wids['edge'] = Choice(panel, choices=edges, size=(60, -1))
+        self.wids['atsym']  = Choice(panel, choices=atsyms, size=(75, -1))
+        self.wids['edge']   = Choice(panel, choices=edges, size=(60, -1))
 
         self.wids['is_frozen'] = Check(panel, default=False, label='Freeze Group',
                                        action=self.onFreezeGroup)
@@ -167,7 +182,7 @@ class XASNormPanel(TaskPanel):
         HLINEWID = 575
         panel.Add(SimpleText(panel, 'XAS Pre-edge subtraction and Normalization',
                              size=(350, -1), **self.titleopts), style=LEFT, dcol=4)
-        
+
         panel.Add(SimpleText(panel, 'Copy to Selected Groups:'),
                   style=RIGHT, dcol=2)
 
@@ -177,7 +192,8 @@ class XASNormPanel(TaskPanel):
         panel.Add(plot_voff, style=RIGHT)
 
         panel.Add(plot_one, newrow=True)
-        panel.Add(self.plotone_op, dcol=4)
+        panel.Add(self.plotone_op, dcol=3)
+        panel.Add(self.plot_erange, dcol=1)
         panel.Add(CopyBtn('plotone_op'), dcol=1, style=RIGHT)
 
         panel.Add(HLine(panel, size=(HLINEWID, 3)), dcol=6, newrow=True)
@@ -280,7 +296,7 @@ class XASNormPanel(TaskPanel):
 
         setattr(dgroup, self.configname, conf)
         return conf
-    
+
     def fill_form(self, dgroup):
         """fill in form from a data group"""
         opts = self.get_config(dgroup)
@@ -408,7 +424,14 @@ class XASNormPanel(TaskPanel):
     def onFreezeGroup(self, evt=None):
         self._set_frozen(evt.IsChecked())
 
+    def onPlotEither(self, evt=None):
+        if self.last_plot_type == 'multi':
+            self.onPlotSel(evt=evt)
+        else:
+            self.onPlotOne(evt=evt)
+
     def onPlotOne(self, evt=None):
+        self.last_plot_type = 'one'
         self.plot(self.controller.get_group())
 
     def onVoffset(self, evt=None):
@@ -417,6 +440,7 @@ class XASNormPanel(TaskPanel):
 
     def onPlotSel(self, evt=None):
         newplot = True
+        self.last_plot_type = 'multi'
         group_ids = self.controller.filelist.GetCheckedStrings()
         if len(group_ids) < 1:
             return
@@ -616,11 +640,12 @@ class XASNormPanel(TaskPanel):
             return
         form = self.read_form()
         self.process(dgroup=dgroup)
-        self.plot(dgroup)
+        self.onPlotEither()
 
     def make_dnormde(self, dgroup):
         form = dict(group=dgroup.groupname)
         self.larch_eval("{group:s}.dnormde={group:s}.dmude/{group:s}.edge_step".format(**form))
+        self.larch_eval("{group:s}.d2normde={group:s}.d2mude/{group:s}.edge_step".format(**form))
 
     def process(self, dgroup=None, force_mback=False, noskip=False, **kws):
         """ handle process (pre-edge/normalize) of XAS data from XAS form
@@ -713,7 +738,6 @@ class XASNormPanel(TaskPanel):
 {group:s}.edge_step = 1.0*{group:s}.edge_step_{normmeth:s}"""
             self.larch_eval(expr.format(**form))
 
-
         self.make_dnormde(dgroup)
 
         if form['auto_e0']:
@@ -754,6 +778,7 @@ class XASNormPanel(TaskPanel):
             dgroup.plot_ylabel = 'y'
             dgroup.plot_yarrays = [('ydat', PLOTOPTS_1, 'ydat')]
             dgroup.dmude = np.gradient(dgroup.ydat)/np.gradient(dgroup.xdat)
+            dgroup.d2mude = np.gradient(dgroup.dmude)/np.gradient(dgroup.xdat)
             if not hasattr(dgroup, 'scale'):
                 dgroup.scale = 1.0
 
@@ -761,6 +786,9 @@ class XASNormPanel(TaskPanel):
             if pchoice == 'dmude':
                 dgroup.plot_ylabel = 'dy/dx'
                 dgroup.plot_yarrays = [('dmude', PLOTOPTS_1, 'dy/dx')]
+            elif pchoice == 'd2mude':
+                dgroup.plot_ylabel = 'd2y/dx2'
+                dgroup.plot_yarrays = [('d2mude', PLOTOPTS_1, 'd2y/dx')]
             elif pchoice == 'norm':
                 dgroup.plot_ylabel = 'scaled y'
                 dgroup.plot_yarrays = [('norm', PLOTOPTS_1, 'y/scale')]
@@ -769,12 +797,17 @@ class XASNormPanel(TaskPanel):
                 dgroup.plot_y2label = 'dy/dx'
                 dgroup.plot_yarrays = [('ydat', PLOTOPTS_1, 'y'),
                                        ('dnormde', PLOTOPTS_D, 'dy/dx')]
+            elif pchoice == 'norm+d2normde':
+                lab = plotlabels.norm
+                dgroup.plot_y2label = 'd2y/dx2'
+                dgroup.plot_yarrays = [('ydat', PLOTOPTS_1, 'y'),
+                                       ('d2normde', PLOTOPTS_D, 'd2y/dx')]
             return
 
-        req_attrs = ['e0', 'norm', 'dmude', 'pre_edge']
+        req_attrs = ['e0', 'norm', 'dmude', 'd2mude', 'pre_edge']
 
         pchoice = PlotOne_Choices[self.plotone_op.GetStringSelection()]
-        if pchoice in ('mu', 'norm', 'flat', 'dmude'):
+        if pchoice in ('mu', 'norm', 'flat', 'dmude', 'd2mude'):
             lab = getattr(plotlabels, pchoice)
             dgroup.plot_yarrays = [(pchoice, PLOTOPTS_1, lab)]
 
@@ -793,6 +826,12 @@ class XASNormPanel(TaskPanel):
             dgroup.plot_yarrays = [('mu', PLOTOPTS_1, lab),
                                    ('dmude', PLOTOPTS_D, lab2)]
             dgroup.plot_y2label = lab2
+        elif pchoice == 'mu+d2mude':
+            lab = plotlabels.mu
+            lab2 = plotlabels.d2mude
+            dgroup.plot_yarrays = [('mu', PLOTOPTS_1, lab),
+                                   ('d2mude', PLOTOPTS_D, lab2)]
+            dgroup.plot_y2label = lab2
 
         elif pchoice == 'norm+dnormde':
             lab = plotlabels.norm
@@ -800,6 +839,12 @@ class XASNormPanel(TaskPanel):
             dgroup.plot_yarrays = [('norm', PLOTOPTS_1, lab),
                                    ('dnormde', PLOTOPTS_D, lab2)]
             dgroup.plot_y2label = lab2
+        elif pchoice == 'norm+d2normde':
+            lab = plotlabels.norm
+            lab2 = plotlabels.d2mude + ' (normalized)'
+            dgroup.plot_yarrays = [('norm', PLOTOPTS_1, lab),
+                                   ('d2normde', PLOTOPTS_D, lab2)]
+            dgroup.plot_y2label = lab2
 
         elif pchoice == 'mback_norm':
             req_attrs.append('mback_norm')
@@ -847,6 +892,9 @@ class XASNormPanel(TaskPanel):
         if new:
             plotcmd = ppanel.plot
 
+        erange = Plot_EnergyRanges[self.plot_erange.GetStringSelection()]
+        self.controller.set_plot_erange(erange)
+
         groupname = getattr(dgroup, 'groupname', None)
         if groupname is None:
             return
@@ -859,6 +907,7 @@ class XASNormPanel(TaskPanel):
              getattr(dgroup, 'mu', None) is None or
              getattr(dgroup, 'e0', None) is None or
              getattr(dgroup, 'dmude', None) is None or
+             getattr(dgroup, 'd2mude', None) is None or
              getattr(dgroup, 'norm', None) is None)):
             self.process(dgroup=dgroup)
         self.get_plot_arrays(dgroup)
@@ -883,6 +932,10 @@ class XASNormPanel(TaskPanel):
             popts['ymin'] = viewlims[2]
             popts['ymax'] = viewlims[3]
 
+        if erange is not None and hasattr(dgroup, 'e0'):
+            popts['xmin'] = dgroup.e0 + erange[0]
+            popts['xmax'] = dgroup.e0 + erange[1]
+
         popts['xlabel'] = dgroup.plot_xlabel
         popts['ylabel'] = dgroup.plot_ylabel
         if getattr(dgroup, 'plot_y2label', None) is not None:
@@ -914,12 +967,13 @@ class XASNormPanel(TaskPanel):
         narr = len(plot_yarrays) - 1
         for i, pydat in enumerate(plot_yarrays):
             yaname, yopts, yalabel = pydat
+            # print(" PLOT :: ", i, pydat)
             popts.update(yopts)
             if yalabel is not None:
                 popts['label'] = yalabel
 
             popts['delay_draw'] = delay_draw or (i != narr)
-            if yaname == 'dnormde' and not hasattr(dgroup, yaname):
+            if yaname in ('dnormde', 'd2normde') and not hasattr(dgroup, yaname):
                 self.make_dnormde(dgroup)
             if yaname == 'norm_mback' and not hasattr(dgroup, yaname):
                 self.process(dgroup=dgroup, noskip=True, force_mback=True)
@@ -931,7 +985,7 @@ class XASNormPanel(TaskPanel):
             axes = ppanel.axes
             for etype, x, y, opts in plot_extras:
                 if etype == 'marker':
-                    xpopts = {'marker': 'o', 'markersize': 4,
+                    xpopts = {'marker': 'o', 'markersize': 6,
                               'label': '_nolegend_',
                               'markerfacecolor': 'red',
                               'markeredgecolor': '#884444'}
